#-*- coding: utf-8 -*-
from email.mime.base import MIMEBase

from django.core.files.base import ContentFile, File
from django.core.mail.backends.smtp import EmailBackend as base_backend
from django.utils.encoding import smart_text

from ..models import Email, Attachment


class EmailBackend(base_backend):

    def _send(self, email_message):
        sent = super(EmailBackend, self)._send(email_message)
        message_data = {
            'from_email': u'%s' % email_message.from_email,
            'to_emails': u', '.join(email_message.to),
            'cc_emails': u', '.join(email_message.cc),
            'bcc_emails': u', '.join(email_message.bcc),
            'all_recipients': u', '.join(email_message.recipients()),
            'subject': u'%s' % email_message.subject,
            'body': u'%s' % email_message.body,
            'raw': u'%s' % smart_text(email_message.message().as_string())
        }
        email = Email.objects.create(**message_data)
        if hasattr(email_message, 'alternatives'):
            for alternative in email_message.alternatives:
                body, mimetype = alternative
                email.alternatives.create(body=body, type=mimetype)
        if hasattr(email_message, 'attachments'):
            for attachment in email_message.attachments:
                if isinstance(attachment, tuple):
                    filename, content, mimetype = attachment
                    django_file = ContentFile(content, filename)
                elif isinstance(attachment, MIMEBase):
                    filename = attachment.get_filename()
                    django_file = File(attachment, filename)
                    mimetype = None
                email_attachment = Attachment(
                    email=email, file_name=filename, mimetype=mimetype)
                email_attachment.file.save(filename, django_file, save=True)
        return sent



from django.contrib import admin

from.models import Alternative, Email, Attachment


class AlternativesInlineAdmin(admin.StackedInline):
    model = Alternative
    extra = 0
    max_num = 0
    fields = ('body_frame', )

    def get_readonly_fields(self, request, obj=None):
        fields = ['body_frame', ]
        if obj:
            fields += [f.name for f in self.model._meta.fields]
        return fields


class AttachmentInlineAdmin(admin.TabularInline):
    model = Attachment
    extra = 0
    max_num = 0

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return [f.name for f in self.model._meta.fields]
        else:
            return []


class EmailAdmin(admin.ModelAdmin):
    list_display = ('to_emails', 'subject', 'sent_at', )
    search_fields =  ('from_email', 'to_emails', 'subject', 'body', )
    inlines = [AlternativesInlineAdmin, AttachmentInlineAdmin]

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return [f.name for f in self.model._meta.fields]
        else:
            return []


admin.site.register(Email, EmailAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-10 23:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mail_save.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Alternative',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('type', models.CharField(blank=True, default='', max_length=255)),
                ('body', models.TextField(blank=True, default='')),
            ],
            options={
                'ordering': ('type',),
            },
        ),
        migrations.CreateModel(
            name='Attachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file_name', models.CharField(max_length=255)),
                ('file', models.FileField(upload_to=mail_save.models.email_attachment_path)),
                ('mimetype', models.CharField(blank=True, default=None, max_length=255, null=True)),
            ],
            options={
                'ordering': ('file_name',),
            },
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('from_email', models.CharField(blank=True, default='', max_length=255)),
                ('to_emails', models.TextField(blank=True, default='')),
                ('cc_emails', models.TextField(blank=True, default='')),
                ('bcc_emails', models.TextField(blank=True, default='')),
                ('all_recipients', models.TextField(blank=True, default='')),
                ('headers', models.TextField(blank=True, default='')),
                ('subject', models.TextField(blank=True, default='')),
                ('body', models.TextField(blank=True, default='')),
                ('raw', models.TextField(blank=True, default='')),
                ('sent_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'ordering': ('-sent_at',),
            },
        ),
        migrations.AddField(
            model_name='attachment',
            name='email',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='mail_save.Email'),
        ),
        migrations.AddField(
            model_name='alternative',
            name='email',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='alternatives', to='mail_save.Email'),
        ),
    ]



from django.conf.urls import patterns, url
from .views import EmailAlternativeView


urlpatterns = patterns(
    '',
    url(r'^email_alternative/(?P<pk>\d+)/$',
        EmailAlternativeView.as_view(),
        name='email_alternative'),
)


#-*- coding: utf-8 -*-
from django.db import models
from django.template.loader import render_to_string
from django.utils.html import escape
from django.utils.safestring import mark_safe


class Email(models.Model):
    from_email = models.CharField(blank=True, default='', max_length=255)
    to_emails = models.TextField(blank=True, default='')
    cc_emails = models.TextField(blank=True, default='')
    bcc_emails = models.TextField(blank=True, default='')
    all_recipients = models.TextField(blank=True, default='')
    headers =  models.TextField(blank=True, default='')
    subject = models.TextField(blank=True, default='')
    body = models.TextField(blank=True, default='')
    raw = models.TextField(blank=True, default='')
    sent_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ('-sent_at',)

    def __unicode__(self):
        return u'Email sent from "%s" to "%s" about "%s"' % (
            self.from_email, self.to_emails, self.subject)

    def __str__(self):
        return self.__unicode__()


class Alternative(models.Model):
    email = models.ForeignKey(Email, related_name='alternatives')
    type = models.CharField(blank=True, default='', max_length=255)
    body = models.TextField(blank=True, default='')

    class Meta:
        ordering = ('type',)

    def __unicode__(self):
        return self.type

    def __str__(self):
        return self.__unicode__()

    def body_frame(self):
        if self.id:
            rendered = render_to_string(
                'mail_save/partials/iframe.html',
                {"pk": self.id})
            return rendered


def email_attachment_path(instance, filename):
    return 'mail_save/{0}/{1}'.format(instance.email.id, filename)


class Attachment(models.Model):
    email = models.ForeignKey(Email, related_name='attachments')
    file_name = models.CharField(max_length=255)
    file = models.FileField(upload_to=email_attachment_path)
    mimetype = models.CharField(max_length=255, null=True, blank=True,
                                default=None)

    class Meta:
        ordering = ('file_name',)

    def __unicode__(self):
        return self.file_name

    def __str__(self):
        return self.__unicode__()



from setuptools import setup


setup(name='django_mail_save',
      version='0.0.5',
      packages=['mail_save'],
      description='Save outgoing mail to the database',
      long_description=open('README.rst').read(),
      url='https://github.com/bigmassa/django_mail_save',
      author='Stuart George',
      author_email='stuart.bigmassa@gmail.com',
      license='MIT',
      keywords = ['mail', 'save', 'email', 'admin'],
      install_requires=[
            'Django>=1.8,<1.9.99',
      ],
      include_package_data=True,
      classifiers=[
            'Development Status :: 4 - Beta',
            'Environment :: Web Environment',
            'Framework :: Django',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: MIT License',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3.5',
            'Topic :: System :: Logging',
            'Topic :: Communications :: Email',
      ],
)



from django.views.generic import DetailView

from .models import Alternative


class EmailAlternativeView(DetailView):

    queryset = Alternative.objects.all()
    template_name = 'mail_save/email/alternative.html'
